package xyz.lychee.lagfixer;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.bukkit.Bukkit;
import org.bukkit.event.HandlerList;
import org.bukkit.plugin.java.JavaPlugin;
import xyz.lychee.lagfixer.libs.adventure.platform.bukkit.BukkitAudiences;
import xyz.lychee.lagfixer.managers.CommandManager;
import xyz.lychee.lagfixer.managers.ConfigManager;
import xyz.lychee.lagfixer.managers.ErrorsManager;
import xyz.lychee.lagfixer.managers.HookManager;
import xyz.lychee.lagfixer.managers.MetricsManager;
import xyz.lychee.lagfixer.managers.ModuleManager;
import xyz.lychee.lagfixer.managers.SupportManager;
import xyz.lychee.lagfixer.managers.UpdaterManager;
import xyz.lychee.lagfixer.objects.AbstractManager;
import xyz.lychee.lagfixer.utils.TimingUtil;

public class LagFixer extends JavaPlugin {
   private static LagFixer instance;
   private final ArrayList<AbstractManager> managers = new ArrayList();
   private final Logger logger = new ColoredLogger();
   private BukkitAudiences audiences;

   public void onEnable() {
      instance = this;
      this.audiences = BukkitAudiences.create(this);
      this.getLogger().info("&fRemember to leave a rating!&r &e&l★ ★ ★ ★ ★\n\n\n\n\n\n&fLagFixer - Best Performance Solution!&r\n░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░\n░██╗░░░░░░█████╗░░██████╗░░░███████╗██╗██╗░░██╗███████╗██████╗░░\n░██║░░░░░██╔══██╗██╔════╝░░░██╔════╝██║╚██╗██╔╝██╔════╝██╔══██╗░\n░██║░░░░░███████║██║░░██╗░░░█████╗░░██║░╚███╔╝░█████╗░░██████╔╝░\n░██║░░░░░██╔══██║██║░░╚██╗░░██╔══╝░░██║░██╔██╗░██╔══╝░░██╔══██╗░\n░███████╗██║░░██║╚██████╔╝░░██║░░░░░██║██╔╝╚██╗███████╗██║░░██║░\n░╚══════╝╚═╝░░╚═╝░╚═════╝░░░╚═╝░░░░░╚═╝╚═╝░░╚═╝╚══════╝╚═╝░░╚═╝░\n░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░\n\n\n\n\n");
      this.managers.clear();
      this.loadManager(new ErrorsManager(this));
      this.loadManager(new SupportManager(this));
      this.loadManager(new ConfigManager(this));
      this.loadManager(new HookManager(this));
      this.loadManager(new MetricsManager(this));
      this.loadManager(new UpdaterManager(this));
      this.loadManager(new ModuleManager(this));
      this.loadManager(new CommandManager(this));
      SupportManager.getInstance().getMonitor().start(this.getConfig().getInt("main.monitor_interval"));
      this.getLogger().info("&fRemember to leave a rating!&r &e&l★ ★ ★ ★ ★");
   }

   public void onDisable() {
      for(Iterator<AbstractManager> it = this.managers.iterator(); it.hasNext(); it.remove()) {
         AbstractManager manager = (AbstractManager)it.next();

         try {
            this.getLogger().info("&8(&e" + manager.getClass().getSimpleName() + "&8) &7-> &fDisabling manager...");
            TimingUtil t = TimingUtil.startNew();
            manager.disable();
            this.getLogger().info("&8(&e" + manager.getClass().getSimpleName() + "&8) &7-> &fDisabled manager in &e" + t.stop().getExecutingTime() + "ms&f!");
         } catch (Exception var4) {
            Exception ex = var4;
            this.printError(ex);
         }
      }

      SupportManager.getInstance().getMonitor().stop();
      Bukkit.getScheduler().cancelTasks(this);
      HandlerList.unregisterAll(this);
   }

   public void loadManager(AbstractManager manager) {
      if (manager.isEnabled()) {
         this.managers.add(manager);

         try {
            this.getLogger().info("&8(&e" + manager.getClass().getSimpleName() + "&8) &7-> &fEnabling manager...");
            TimingUtil t = TimingUtil.startNew();
            manager.load();
            this.getLogger().info("&8(&e" + manager.getClass().getSimpleName() + "&8) &7-> &fEnabled manager in &e" + t.stop().getExecutingTime() + "ms&f!");
         } catch (Exception var3) {
            Exception ex = var3;
            this.printError(ex);
         }

      }
   }

   public void printError(Throwable ex) {
      if (ErrorsManager.getInstance() != null && ErrorsManager.getInstance().checkError(ex)) {
         this.getLogger().log(Level.WARNING, ex.getMessage(), ex);
      }

   }

   public ArrayList<AbstractManager> getManagers() {
      return this.managers;
   }

   public Logger getLogger() {
      return this.logger;
   }

   public BukkitAudiences getAudiences() {
      return this.audiences;
   }

   public void setAudiences(BukkitAudiences audiences) {
      this.audiences = audiences;
   }

   public static LagFixer getInstance() {
      return instance;
   }
}
